# -*- coding: utf-8 -*-
"""
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Created on Monday July 04  08:28:00 2018                        %%
%% Last modified:                                                  %%
%% @author: guti0021                                               %%
%%                                                                 %%
%% Script contains three main sections:                            %%
%%     1. Generates new rainfall series                            %%
%%     2. Generates model combos                                   %%
%%     3. Generates folders and files to run model combos          %%
%%                                                                 %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
"""
#------------------------------------------------------------------------------
#----------------------------Import packages-----------------------------------
#------------------------------------------------------------------------------

import os
import shutil
import numpy as np

# Specify model folder location:
folder_path = os.getcwd()
folder_name = os.path.basename(os.path.normpath(folder_path))

#------------------------------------------------------------------------------
#----------------------------Simulations Set up -------------------------------
#------------------------------------------------------------------------------

'''
----------------------------GENERATE RAINFALL SERIES---------------------------
 1. Generate list of rain fluxes and times
 Uses the same total amount applied at different flux rates over
 different periods.
     - Three rain durationsare: 1, 6 and 24 hours
     - Three intesities for each duration corresponding to the following
        annual exceedance probabilities (AEP): extreme(0.2%), rare (2%),
        and a common rate (50%)
     - The pulses are to specify the start and finish times for the
        precipitation applications

-------------------------------------------------------------------------------
'''

rain_duration = [3600, 3600, 3600, 21600, 21600, 21600, 86400, 86400, 86400]

rain_intensity = [5.16e-06, 1.38e-05, 1.93e-05, 1.32e-06, 3.59e-06, 4.54e-06, \
                  4.72e-07, 1.26e-06, 1.91e-06]

time_table_pulses={ 3600:[0, 3600, 90000, 93600, 180000, 183600],\
                   21600:[0, 21600, 108000, 129600, 216000, 237600],\
                   86400:[0, 86400, 172800, 259200, 345600, 432000]}


'''
------------------------INITIAL HEAD AND FLUID TRANSFER------------------------
 1.Depth values from 0 to 3 meters at 5 cm intervals
   Fluid transfer gradient will be set 0.2 m below initial head
-------------------------------------------------------------------------------
'''
# Range of initial GW heads we are testing
depth_range = np.around((np.arange(0., 3., 0.05)), decimals=2).tolist()

fluid_transfer_head = [round(3-(i + 0.2), 2) for i in depth_range]

# Create a list with the initial head files to access them and assign them to
# the model scenarios

Hfolder_name = '3_initial_head_files'
HDir = os.path.join(folder_path, Hfolder_name)
head_output_files = [ i for i in os.listdir(HDir)]
head_output_files.sort()

'''
--------------GENERATE FOLDERS AND FILES TO RUN MODEL COMBOS-------------------
1. Set up a loop that generates a folder for each model combo and
   create or copy all the neccesary files to run phgs
   NOTE: Files are created and numbered in order of appearance in Grok
2. Generate files for the HMC routines
-------------------------------------------------------------------------------
'''
rootDir = (os.getcwd())
gbDir = os.path.join(rootDir, '1_gb')
mpDir = os.path.join(rootDir, '2_mprops')

sub_folders_list = []

# Set up the range of initial heads to run i.e. which models to run
# this list needs to correspond to the different heads under the depth range

test_range = depth_range

models = ['GWdepth' + '_' + str(round((depth_range[i] * 100)))
                            for i in range (len(test_range))]

# Specify which rainfall scenario to test accordingly (scenarios 0-8)
rain_idx = 0

for i in range(len(test_range)):
    model_name = models[i]
    model_path = os.path.join(rootDir, 'model_' + model_name)
    print (model_path)
    sub_folders_list += [model_path]


    #---Copy gb folder
    shutil.copytree(gbDir, (os.path.join(model_path, '1_gb')))

    #---Copy mprops folder
    shutil.copytree(mpDir, (os.path.join(model_path, '2_mprops')))

    #---Copy initial head file
    shutil.copy (os.path.join(HDir, head_output_files[i]),
                 (os.path.join(model_path, head_output_files[i])))



    #--------------Copy all necessary files to run model[i] -------------------
    #--------------------------------------------------------------------------
    for file in os.listdir(rootDir):
        file_src = os.path.join(rootDir,file)
        file_dst = os.path.join(model_path, file)

        if os.path.isfile(file_src):
            shutil.copy(file_src, file_dst)

    os.chdir(model_path)


    #--------------Generate phgs batch and slurm batch scripts ----------------
    #--------------to run models in the super computer-------------------------

    with open('batch.pfx', 'w') as f:
        f.write('abdul')

    with open('00_batch_script.sub', 'w') as f:
        f.write(
        '''#!/bin/bash

### Job Name
#SBATCH --job-name=model_{1}

### Set email type for job
### Accepted options: NONE, BEGIN, END, FAIL, ALL
#SBATCH --mail-type=FAIL

### email address for user
#SBATCH --mail-user=karina.gutierrez@flinders.edu.au

### Queue name that job is submitted to
#SBATCH --partition=tango

### Request nodes
#SBATCH --ntasks=1
#SBATCH --mem=8gb
#SBATCH --time=24:00:00

echo Running on host `hostname`
echo Time is `date`

#Load module(s) if required
#module load application_module

# Setup model run with grok.x

grok.x

# Copy files to scratch and run phgs.x

rm -Rf /scratch/${{USER}}/{2}/model_{1}
mkdir -p /scratch/${{USER}}/{2}/model_{1}

cp -Rf {0}/model_{1} /scratch/${{USER}}/{2}

cd /scratch/${{USER}}/{2}/model_{1}

phgs.x

cp abdulo.water_balance.dat {0}/model_{1}/
cp abdulo.hmc_Outlet_hydrograph.dat {0}/model_{1}/
cp abdulo.hmc_Upstream_hydrograph.dat {0}/model_{1}/
cp abdulo.lst {0}/model_{1}/

#cp ./* {0}/model_{1}/

#Clean up after
rm -Rf /scratch/${{USER}}/{2}/model_{1}
        '''.format(folder_path, model_name, folder_name, ))

    #-----------------------1.Generate Initial head file-----------------------
    #--------------------------------------------------------------------------

    initial_head = 3 - depth_range[i]

    with open('1_set_initial_head.txt', 'w') as f:

        f.write ('initial head from output file\n')
        f.write (head_output_files[i])


    #------------------------2.Fluid transfer file-----------------------------
    #--------------------------------------------------------------------------
    gradient = fluid_transfer_head[i]
    with open ('2_set_fluid_tranfer_gradient.txt', 'w') as f:
        if gradient <= 0:
            f.write ('0.0 0.0')
        else:
            f.write ('0.0' + ' ' + str(gradient))

    #------------------------2.Generate rainfall file--------------------------
    #--------------------------------------------------------------------------

    with open ('2_rain.txt', 'w') as f:
        f.write("{0} {1}".format(time_table_pulses[rain_duration[rain_idx]][0],
                                                    rain_intensity[rain_idx]))
        f.write('\n')
        f.write("{0} {1}".format(time_table_pulses[rain_duration[rain_idx]][1],
                                                    0))
        f.write ('\n')
        f.write("{0} {1}".format(time_table_pulses[rain_duration[rain_idx]][2],
                                                    rain_intensity[rain_idx]))
        f.write('\n')
        f.write("{0} {1}".format(time_table_pulses[rain_duration[rain_idx]][3],
                                                    0))
        f.write ('\n')
        f.write("{0} {1}".format(time_table_pulses[rain_duration[rain_idx]][4],
                                                    rain_intensity[rain_idx]))
        f.write('\n')
        f.write("{0} {1}".format(time_table_pulses[rain_duration[rain_idx]][5],
                                                    0))
        f.write('\n')
        f.write('end')

    #--------------------------3.Output times----------------------------------
    #--------------------------------------------------------------------------

    with open ('3_output_times.txt', 'w') as f:
        for x in range (1,time_table_pulses[rain_duration [rain_idx]][-1]
                                                                + 43200, 1800):
            f.write(str(x))
            f.write('\n')
        f.write('end')

    #--------------------------Generate model summary--------------------------
    #--------------------------------------------------------------------------

    with open('0_model_parameters.txt', 'w') as f:
        f.write('Testing different GW heads --- Model ' + str(i) + '\n'
                + '\n'
                + 'GW =  {} m'.format(depth_range[i])
                + '\n'
                + 'Rainfall rate = {} m/s for {}'.format(rain_intensity
                                        [rain_idx], rain_duration[rain_idx]))



    os.chdir(rootDir)  # Reset cwd for testing

#--------------------------------------------------------------------------
# Write files to automate running the created models
#--------------------------------------------------------------------------


with open("folders.txt", 'w') as f:
    f.write("\n".join(sub_folders_list))

with open("run_all.py", 'w') as f:
    f.write("""
import subprocess

with open('folders.txt', 'r') as f:
    folders = f.readlines()

for folder in folders:
    arguments = \"""
cd {{0}}
sbatch 00_batch_script.sub
\""".format(folder)
    subprocess.call(arguments , shell=True)
    """.format(folder_name))
